/*

  Reflow-Temperatur-Regelung V0.1-beta
  Copyright 2006 Thomas Pfeifer

  http://thomaspfeifer.net/backofen_smd_reflow.htm

  Commercial use prohibited


*/

//#define oldgcc

#include <avr/io.h>
#include <stdio.h>
#include <stdlib.h>

#define F_CPU 8000000UL  // 8 MHz
#define BAUD_RATE 38400


#ifdef oldgcc
  #include <avr/delay.h>
#else
  #include <util/delay.h>
#endif

void adcinit();
int getadc(uint8_t channel);
void waitms(int ms);
void error();
void control(int t_ist, int t_soll);
int updateProfile(int t);

int profile_t[10];
int profile_T[10];

static int uart_putchar(char c)
//############################################################################
{
  if (c == '\n')
    uart_putchar('\r');
  //Warten solange bis Zeichen gesendet wurde
  loop_until_bit_is_set(UCSRA, UDRE);
  //Ausgabe des Zeichens
  UDR = c;
  return (0);
}

void UART_Init (void) {
  //Enable TXEN im Register UCR TX-Data Enable
  UCSRB=(1 << TXEN);
  //Teiler wird gesetzt
  UBRRL=(F_CPU / (BAUD_RATE * 16L) - 1);

#ifdef oldgcc
  //ffnet einen Kanal fr printf (STDOUT)
  fdevopen (uart_putchar, NULL, 0);
  static int uart_putchar(char c, FILE *stream);
#else
  static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL,_FDEV_SETUP_WRITE);
  stdout = &mystdout;
#endif
}

int convert2degree(int value) {
  static const float c1=19.5;
  static const float v1=268;  //ADC-Wert=264 @ 20C

  static const float c2=222;
  static const float v2=125;  //ADC-Wert=134 @ 208C


  float m=((c1-c2)/(v1-v2));
  return((int)((m*(float)value)+(-m*v1)+c1));
  //return((int)((-m*v1)+c1));
}




int main (void) {
  //*********************
  //* Altera BGA Profil *
  //*********************
  profile_t[0]=0;
  profile_T[0]=0;

  profile_t[1]=100;
  profile_T[1]=150;

  profile_t[2]=210;
  profile_T[2]=183;

  profile_t[3]=220;
  profile_T[3]=210;

  profile_t[4]=250;
  profile_T[4]=210;

  profile_t[5]=260;
  profile_T[5]=0;

  profile_t[6]=1000;
  profile_T[6]=0;
  //*********************

  //*********************
  //* 50C *
  //*********************
  /*
  profile_t[0]=0;
  profile_T[0]=40;

  profile_t[1]=1000;
  profile_T[1]=40;
  */
  //*********************

  int adc;
  int tick=0;
  int t=0;
  float t_soll;

  adcinit();
  UART_Init();
  printf("ok\n");

  DDRC  = 0b00000000;

  DDRB  = 0b00000000;
  PORTB = 0b00000000;

  DDRD  = 0b11100000;  // 1=Output, 0=Input
  PORTD = 0b00000000;  //off



  t_soll=0;
  t=0;
  while (t<1000) {
    tick++;

    // ADC auslesen
    adc=getadc(0);
    if (adc<100 || adc>1000) error(); // sensor break/short detect

    // Regelung
    control(convert2degree(adc),t_soll);

    // 1/s t_soll update
    if (tick==50) {
      tick=0;
      t_soll=updateProfile(t);
      if (t_soll<-5 || t_soll>300) error();

      printf("%d,%d,%d\n",adc,(int)t_soll,convert2degree(adc));
      t++;
    }
    waitms(20);
  }

  PORTD = 0b00000000;  //off
  while (1==1) {}


}

void error() {
  while(1==1) {
    PORTD = 0b10000000;  //on
    waitms(600);
    PORTD = 0b00000000;  //off
    waitms(600);
    printf("Error\n");
  }
}
void control(int t_ist, int t_soll) {
  if (t_ist>t_soll) {
    PORTD = 0b00000000;  //off
  } else {
    PORTD = 0b11100000;  //on
  }
}


float profil_t;
int profil;
float dt;
int updateProfile(int t) {
  if (t==0) {
    profil=0;
  }
  if (t==profile_t[profil]) {
    profil_t=profile_T[profil];

    dt=((float)profile_T[profil+1]-(float)profile_T[profil])/((float)profile_t[profil+1]-(float)profile_t[profil]);
    //printf("profil: %d\n",profil);
    profil++;
  } else {
    profil_t+=dt;
  }

  return((int)profil_t);
}
void adcinit() {
  // Activate ADC with Prescaler 16 --> 1Mhz/16 = 62.5kHz
  //ADCSRA = _BV(ADEN) | _BV(ADPS2);
  ADCSRA = _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1);
  ADMUX=  0;
}

int getadc(uint8_t channel) {
  // Select pin ADC0 using MUX
  ADMUX = channel | _BV(REFS0) | _BV(REFS1);

  //Start conversion
  ADCSRA |= _BV(ADSC);

  // wait until converstion completed
  while (ADCSRA & _BV(ADSC) ) {}

    // get converted value
  return(ADCW);
}

void waitms(int ms) {
  int i;
  for (i=0;i<ms;i++) _delay_ms(1);
}

